@@ -17,9 +17,13 @@ import sklearn.linear_model
 import sklearn.tree
 import sklearn.ensemble
 import sklearn.metrics
+from sklearn.metrics import plot_confusion_matrix, classification_report
+
 import sklearn.naive_bayes
 import sklearn.neighbors
 
+import seaborn as sns
+import matplotlib.pyplot as plt
 import pandas as pd
 from predictions_analyzer.analyze import Analyzer
 
@@ -209,7 +213,9 @@ class ClassificationAnalyzer():
         self.X_valid = X_valid
         self.y_valid = y_valid
 
-    def split_val_train(self, verbose = True):
+    def split_val_train(self,
+                        train_fraction = 4/5,
+                        verbose = True):
         """
         Splits data into train and validation splits.
         Useful for quick processing.
@@ -222,7 +228,7 @@ class ClassificationAnalyzer():
         y = self.y
 
         length_of_X = len(X)
-        split_at_id = (length_of_X * 2) // 3
+        split_at_id = int(length_of_X * train_fraction)
 
         self.X_train = X.iloc[0:split_at_id, :]
         self.y_train = y.iloc[0:split_at_id]
@@ -397,9 +403,87 @@ class ClassificationAnalyzer():
         """
         pass
 
-    def do_binary_metrics(self):
+
+    def show_confusion_matrix(self, verbose = True):
+
+        for model, model_name in tqdm(self.models):
+
+            # TODO: Add Exception handling to predict doesn't stop.
+
+            disp = plot_confusion_matrix(model,
+                                  self.X_valid,
+                                  self.y_true,
+                                  cmap = plt.cm.Blues)
+
+            plt.title("Confusion Matrix For: " + model_name)
+            plt.show()
+
+
+    def show_classification_report(self):
+
+        for col in self.preds_df.columns:
+
+            clf_report = classification_report(self.y_true, self.preds_df.loc[:,col], output_dict=True)
+            clf_report_print = classification_report(self.y_true, self.preds_df.loc[:, col])
+
+            print("\n\n" + col)
+            print(clf_report_print)
+            sns.heatmap(pd.DataFrame(clf_report).T,
+                        annot = True, vmin = 0, vmax = 1,
+                        cmap = plt.cm.Blues)
+
+            plt.title("Classification Report for: " + col)
+            plt.show()
+
+    def show_best_recall(self):
         pass
 
+    def show_best_precision(self):
+        pass
+
+    def show_best_f1(self):
+        pass
+
+    def show_best_on_hardest_samples(self):
+        pass
+
+    def show_confidence_on_hardest_samples(self):
+        pass
+
+    def get_best_on_particular_rows(self):
+        pass
+
+    def ensemble_analysis(self):
+        # Which ensemble gives the best results?
+
+        # Create different combinations of all the preds
+        # Add the ensembles to the confusion matrices / classification reports!
+        # Voting classifiers
+        # Stacking Classifiers
+        # Mean and Median Classifiers
+
+        pass
+
+    def show_all_reports(self):
+
+        for col, model in zip(self.preds_df.columns, self.models):
+
+            assert(model[1] == col)  # The column name must equal the model name
+
+            # plt.subplots - Confusion matrix and Classification report
+            # Speed
+
+        # Overall Reports:
+        # Best Precision
+        # Best Recall
+        # Best f1 Score.
+        # Best Accuracy.
+
+        # Diversity Analysis.
+
+
+    def do_binary_metrics(self):
+        pass
 
     def get_num_wrong_right(self):
         """
@@ -477,20 +561,29 @@ class ClassificationAnalyzer():
 
         self.correct_mask = correct_mask
 
-        n_correct = correct_mask.sum(axis = 1).sort_values()
+        n_correct = correct_mask.sum(axis = 1)
 
-        self.n_correct = n_correct
+        mask_with_margins = correct_mask.insert(0, "n_clf_correct", n_correct)
 
-        print("\nSorted Hardest Samples: retrievable with the ._hardest_samples attribute")
+        self.correct_mask_with_margins_ = mask_with_margins
+
+        n_correct = n_correct.sort_values()
+        self.hardest_samples_ = n_correct
+
+        print("\nSorted Hardest Samples: retrievable with the .hardest_samples_ attribute")
         print("Key is row index of sample, Value is the number of correct from all predictors")
         print(n_correct)
 
-        print("\nNumber of correct: retrievable with the ._n_correct attribute")
+        self.n_freq_correct = n_correct.value_counts().sort_values()
+
+        print("\nNumber of correct: retrievable with the .n_freq_correct attribute_")
         print("Index is the number of correct predictions, value is how many samples had that number of correct")
-        print(n_correct.value_counts().sort_values())
+        print(self.n_freq_correct)
 
-        return n_correct
+        print("\nFull Mask: retrievable with the .correct_mask_with_margins_ attribute")
+        print(mask_with_margins)
 
+        print(self.correct_mask)
 
     def cluster_wrong_answers(self):
         """
